[9:04 PM] Gulizia, Joe
/* Introduction

This is the line following robot code for KU SP24 ME608 Project 1 coded by the group titled Mason and the J's.

Members: Mason Veeder, Joe Koscal, Joshua Stove, Jamal Robinson, Joe Gulizia

==========================================================================================

Much of this code is influenced by provided Canvas resources and guided by advice from Dr. Wilson, her chatBot, and her GTA, Nafiseh Aftah.

Additionally, the GitHub page for the IRremote library helped for specific arguments and syntax for the IR receiver.

IR Receiver GitHub link: https://github.com/Arduino-IRremote/Arduino-IRremote?tab=readme-ov-file#receiving-ir-codes

Outside article for IR Receiver link: https://www.makerguides.com/ir-receiver-remote-arduino-tutorial/ 

Above article provided some help in understanding the originally complex GitHub documentation.
 
It should be noted the Joe G found an extra remote from his projector and the below code is with that remote in mind.

The codes/remote can be reassigned easily in the switch-case statements.

Simply run a basic IR Decoder to check what each button's command is if choosing a new remote.

*/
 
/* Proposed Updates

[] Map the Menu button to set speed back to a defaultSpeed: if(setDefaultSpeed {speed = defaultSpeed;})

[] Have an led turn on when motors are out of standby mode meaning they're on or ready to run

[] Have leds display line detection status

*/
 
#include <IRremote.hpp> // The IR receiver library "IRremote" by shirriff, z3t0, ArminJo. Version 4.3.1 tested on Arduino IDE 2.3.2

#define IR_RECEIVE_PIN 2 // The pin the IR receiver is connected to
 
// General values and parameters

unsigned long inputCommand; // Storage value for IR commands

int debuggingOn = true; // If true: Prints button names in order to verify commands are received and switch-cases are running

int inputDelay = 5; // Delay in taking new IRData

int movementDuration = 200; // This is how long a motor will stay on when an arrow key is pressed in manual mode

int turnDuration = 100; // Similar to the above variable but this is a shorter delay to allow more precision while turning 

int lineFollowDelay = 10; // Affects the frequency in which line sensor data is processed and acted on.

int lineFollowOn = false; // lineFollower mode toggle variable, initially off

int roombaDelay = 50; // Sets the delay between measurements in roomba mode

int roombaModeOn = false; // roombaMode toggle variable, initially off

int speed = 100; // Initial speed value for motors, put the best general speed here. Range is 40 to 200 (may not be current)

const int ledPin = A5; // Pin for the testing led mentioned in the next line

int ledOn = false; // This is tied to the "Mute" button for physically testing if the robot is receiving signals. Also shows arduino crashes

int standbyOn = true; // The motors are in standby mode by default, this bool is for the "Power" button switch-case statement below
 
// Motor pins listed in order as they are on the breakout board

const int motorAPWM = 6;  // Pin for PWM control for motorA speed **cannot be pin 11, breaks code when speed < 100 ** same for pin 3 speed > 100

const int motorAIn2 = 9; 

const int motorAIn1 = 8; 

const int standbyPin = 7; // Standby pin

const int motorBIn1 = 4; 

const int motorBIn2 = 3; 

const int motorBPWM = 5;  // Pin for PWM control for motorB speed

// Pins for the left and right line sensors

const int leftLineSensor = A0; // Analog pin for left line sensor

const int rightLineSensor = A1; // Analog pin for right line sensor

// Pins for the sonic distance sensor

const int trigpin = 10; // OUTPUT for the sonic distance sensor

const int echopin = 11; // INPUT  for the sonic distance sensor
 
void setup() {

  Serial.begin(9600); // Begin Serial monitor

  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK); // Start the receiver

  pinMode(ledPin, OUTPUT); // Testing led pinMode
 
  // Motor pinModes

  pinMode(motorAIn1, OUTPUT);

  pinMode(motorAIn2, OUTPUT);

  pinMode(motorAPWM, OUTPUT);

  pinMode(motorBIn1, OUTPUT);

  pinMode(motorBIn2, OUTPUT);

  pinMode(motorBPWM, OUTPUT);

  pinMode(standbyPin, OUTPUT);
 
  // Line sensor pinModes

  pinMode(leftLineSensor, INPUT);

  pinMode(rightLineSensor, INPUT);
 
  // Ultrasonic distance sensor pinModes

  pinMode(trigpin, OUTPUT);

  pinMode(echopin, INPUT);  

}
 
void loop() {

  if (IrReceiver.decode() && IrReceiver.decodedIRData.protocol == 8) { // When IR signal is detected from NEC device (Joe G's personal remote)

    inputCommand = IrReceiver.decodedIRData.command; // Stores IR .command data

    if (debugginOn) { // Commented out as remote trouble shooting was no longer necessary

      //Serial.println(inputCommand, HEX); // Troubleshooting print of hex codes from IR receiver

      //Serial.println(IrReceiver.decodedIRData.decodedRawData, HEX); // Print the received decodedRawData

    }

    switch (inputCommand) { // Assigns outputs for each input/button, sorted from top-down left to right on physical remote

      // Note the two slashes: "//" is for buttons that are unassigned and assigned buttons have four slashes: "////"

      case 0xA8: //// Assigned standby mode toggle to power button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("POWER");

        }

        if (standbyOn) { //Takes the motors out of standby mode

          Serial.println("Motors On");

          digitalWrite(standbyPin, HIGH);

          standbyOn = false;

        }

        else if (!standbyOn) { // Puts the motors in standby mode after stopping them

          Stop();

          Serial.println("Motors Off");

          digitalWrite(standbyPin, LOW);

          standbyOn = true;

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x88: //// Assigned test led to mute button

        // Turns an optional test led on or off

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("MUTE");

        }

        if (ledOn) {

          analogWrite(ledPin, 0);

          ledOn = false;

        }

        else if (!ledOn) {

          analogWrite(ledPin, 255);

          ledOn = true;

        }

        delay(inputDelay); // Wait between readings

        break;   

      case 0x98: //   Available zoom button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("ZOOM");

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x93: //// Assigned line follower toggle to pause play button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("PAUSE/PLAY");

        }

        if (lineFollowOn) { // Flips the value of lineFollowOn

          lineFollowOn = false;

          Serial.println("PAUSE");

        }

        else if (!lineFollowOn) {

          lineFollowOn = true;

          Serial.println("PLAY");

        }

        delay(inputDelay); // Wait between readings

        Stop();

        break;

      case 0x82: //   Available flip button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("FLIP");

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x95: //// Assigned Forward() to arrow buttton

        // Runs motors to go forward

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("UP");

        }

        Forward(speed);

        delay(movementDuration); // Time the motors are moving forward

        Stop();

        break;

      case 0x9B: //// Assigned Left() to left arrow button

        // Runs motors to turn left

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("LEFT");

        }

        Left(speed);

        delay(turnDuration); // Time the motors are moving forward

        Stop();

        break;

      case 0x99: //// Assigned Right() to right arrow button

        // Runs motors to turn right

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("RIGHT");

        }

        Right(speed);

        delay(turnDuration); // Time the motors are moving forward

        Stop();

        break;

      case 0x9A: //// Assigned Backward() to down arrow button

        // Runs motors to go backwards

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("DOWN");

        }

        Backward(speed);

        delay(movementDuration); // Time the motors are moving forward

        Stop();

        break;

      case 0x9E: //// Assigned Stop() to OK button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("OK");

        }

        Stop();

        delay(inputDelay); // Wait between readings

        break;

      case 0xA4: //   Available back button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("BACK");

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x91: //// Assigned Roomba Mode toggle to menu button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("MENU");

        }

        if (roombaModeOn) { // Flips the value of roombaModeOn

          roombaModeOn = false;

          Serial.println("ROOMBA OFF");

        }

        else if (!roombaModeOn) {

          roombaModeOn = true;

          Serial.println("ROOMBA ON");

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x97: //   Available enter button

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("ENTER");

        }

        delay(inputDelay); // Wait between readings

        break;

      case 0x9C: //// Assigned speed decrease to volume down button

        // Decreases speed down to 40

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("VOLUME DOWN");

        }

        if (speed <= 50 || (IrReceiver.decodedIRData.decodedRawData == 0x0 && speed <= 60)) { // Prevents motor speed from going below 0

          speed = 40;

          Serial.println("MIN SPEED");

        }

        else if (IrReceiver.decodedIRData.decodedRawData == 0x0) { // If Volume Down button is held, double the speed decrease

          speed = speed - 20;

          Serial.println(speed);

        }

        else { // Decrease speed by 10

          speed = speed - 10;

          Serial.println(speed);

        }

        delay(50); // Wait between readings

        break;

      case 0x8C: //// Assigned speed increase to volume up button

        // Increases speed up to 200

        if (debuggingOn) { // Print button name when debuggingOn is true

          Serial.println("VOLUME UP");

        }

        if (speed >= 190 || (IrReceiver.decodedIRData.decodedRawData == 0x0 && speed >= 180)) { // Prevents motor speed from exceeding 250

          speed = 200;

          Serial.println("MAX SPEED");

        }

        else if (IrReceiver.decodedIRData.decodedRawData == 0x0) { // If Volume Up button is held, double the speed increase

          speed = speed + 20;

          Serial.println(speed);

        }

        else { // Increase speed by 10

          speed = speed + 10;

          Serial.println(speed);

        }

        delay(50); // Wait between readings

        break;} // Switch close


    IrReceiver.resume(); // Enable IR receiver for next input from remote

  } // If statement close for IR receiving
 
  else if (IrReceiver.decode() && IrReceiver.decodedIRData.protocol != 8) { 

    // Filters correct commands coming from incorrect device (had issues with iPhone IR blaster on FaceID sensor sending repeat command 0x0)

    // ID 8 means the remote is using the NEC standard like Joe G's personal remote

    Serial.println("Unknown IR Data");

    delay(inputDelay); // Delay in inputs

    IrReceiver.resume(); // Enable IR receiver of the next input from remote

  } // Else If statement close for IR receiving of bad data
 
  if (lineFollowOn) { // Runs if line follower mode is set true in above switch statement

    int leftSensorReading = analogRead(leftLineSensor); // Collect left line follower sensor data

    int rightSensorReading = analogRead(rightLineSensor); // Collect right line follower sensor data

    if (debugginOn) { //Output values to the Serial Monitor for when in debugging mode

      Serial.println(analogRead(leftLineSensor));

      Serial.println(analogRead(rightLineSensor));

    }

    // Both sensors detect no line, move forward

    if (leftSensorReading < 700 && rightSensorReading < 700) {

      if (debuggingOn) {

        Serial.println("Forward");

      }

      Forward(speed);

    }

    // Left sensor detects the line, turn left

    else if (leftSensorReading < 700) {

      if (debuggingOn) {

        Serial.println("Left");

      }

      Left(speed);

    }

    // Right sensor detects the line, turn right

    else if (rightSensorReading < 700) {

      if (debuggingOn) {

        Serial.println("Right");

      }

      Right(speed);

    }

    // When line is detected on both sensors, stop

    else {

      if (debuggingOn) {

        Serial.println("Stop");

      }

      Stop();

    }

    delay(lineFollowDelay); // Can be changed to increase or decrease intake of measurments and length of movements

    Stop(); // Always run stop just in case, almost no delay 

  } // If statement close for Line Following code
 
  else if (roombaModeOn) {

    // The lack of range of measurements is not suitible for our bot, wheels and outside edges often collide

    if (debuggingOn) {

        Serial.println("Roomba Mode");

      }

    while (getDistance() >= 10 && !IrReceiver.decode()) { // When nothing is detected, move forward

      Forward(speed); 

      if (debuggingOn) {

        Serial.println(getDistance());

      }

    }

    Stop(); // Ends the Forward() function upon detection from distance sensor

    while (getDistance() < 10 && !IrReceiver.decode()) { // When something is detected by distance sensor, turn right

      //Serial.println(getDistance());

      speed = 100;

      Right(speed);

      delay(50);

    }

    Stop(); // Ends the Right()function upon no longer detecting an obstacle

    delay(roombaDelay); // Lowers the frequency of inputs

  }
 
} // Void loop close
 
// Functions

// Drive commands that pass a set speed

void Forward(int speed){

  digitalWrite(motorAIn1, LOW);

  digitalWrite(motorAIn2, HIGH);

  digitalWrite(motorBIn1, LOW);

  digitalWrite(motorBIn2, HIGH);

  analogWrite(motorAPWM, speed);

  analogWrite(motorBPWM, speed);}

void Backward(int speed){

  digitalWrite(motorAIn1, HIGH);

  digitalWrite(motorAIn2, LOW);

  digitalWrite(motorBIn1, HIGH);

  digitalWrite(motorBIn2, LOW);

  analogWrite(motorAPWM, speed);

  analogWrite(motorBPWM, speed);}

void Right(int speed){

  digitalWrite(motorAIn1, LOW);

  digitalWrite(motorAIn2, HIGH);

  digitalWrite(motorBIn1, HIGH);

  digitalWrite(motorBIn2, LOW);

  analogWrite(motorAPWM, speed);

  analogWrite(motorBPWM, speed);}

void Left(int speed){

  digitalWrite(motorAIn1, HIGH);

  digitalWrite(motorAIn2, LOW);

  digitalWrite(motorBIn1, LOW);

  digitalWrite(motorBIn2, HIGH);

  analogWrite(motorAPWM, speed);

  analogWrite(motorBPWM, speed);}

void Stop(){

  digitalWrite(motorAIn1, LOW);

  digitalWrite(motorAIn2, LOW);

  digitalWrite(motorBIn1, LOW);

  digitalWrite(motorBIn2, LOW);

  analogWrite(motorAPWM, 0);

  analogWrite(motorBPWM, 0);}

// Collects distance data from the ultrasonic distance sensor, returns in centimeters

float getDistance() {

  digitalWrite(trigpin, LOW);

  delayMicroseconds(2);

  digitalWrite(trigpin, HIGH);

  delayMicroseconds(10);

  digitalWrite(trigpin, LOW);

  float duration = pulseIn(echopin, HIGH);

  float distance = duration / 148; // Calculate distance in CM

  return distance;}
